use crate::ems::bms::bms::Bms;
use crate::ems::pcs::pcs::Pcs;
use crate::ems::service::Device::{BMS, PCS};
use std::collections::HashMap;
use tokio::task::JoinHandle;

pub trait Service {
    fn run(&self) -> Result<JoinHandle<()>, anyhow::Error>;
}

/// 设备类型
#[derive(Eq, Hash, PartialEq)]
pub enum Device<'a> {
    PCS(&'a str),
    BMS(&'a str),
}

pub struct Ctx {}

/// EMS
/// devices: 实现了Service trait的列表
pub struct Ems<'a> {
    pub(crate) devices: HashMap<Device<'a>, Box<dyn Service>>,
    pub context: Ctx,
}

impl Ems<'_> {
    pub async fn new() -> Self {
        let mut devices: HashMap<Device, Box<dyn Service>> = HashMap::new();
        let pcs = Pcs::new().await;
        let bms = Bms::new().await;
        devices.insert(PCS("PCS"), Box::new(pcs));
        devices.insert(BMS("BMS"), Box::new(bms));
        devices.get(&PCS("PCS")).unwrap().run().expect("");
        let context = Ctx {};
        Ems { devices, context }
    }

    pub async fn start_services(&self) -> Result<(), anyhow::Error> {
        let mut handles = Vec::new();
        for (_name, dev) in &self.devices {
            let handle = dev.run()?;
            handles.push(handle);
        }
        for handle in handles {
            handle.await?;
        }
        Ok(())
    }
}
